package com.plannaplan.controllers;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static org.springframework.security.test.web.servlet.setup.SecurityMockMvcConfigurers.*;

import com.plannaplan.entities.User;
import com.plannaplan.services.UserService;
import com.plannaplan.types.UserRoles;

@RunWith(SpringRunner.class)
@SpringBootTest
@ContextConfiguration
public class StatisticsControllerTest extends AbstractControllerTest {

    private static final String GROUP_AMMOUNTS_ENDPOINT = "/api/v1/statistics/groups/created";
    private static final String GROUP_FULL_AMMOUNTS_ENDPOINT = "/api/v1/statistics/groups/full";
    private static final String USER_ASSIGNED_AMMOUNTS_ENDPOINT = "/api/v1/statistics/users/registered";
    private static final String USER_NO_ASSIGNED_AMMOUNTS_ENDPOINT = "/api/v1/statistics/users/noregistered";
    private static final String USER_ACCEPTED_AMMOUNTS_ENDPOINT = "/api/v1/statistics/users/accepted";
    private static final String USER_PARTLY_ACCEPTED_AMMOUNTS_ENDPOINT = "/api/v1/statistics/users/accepted/partly";

    @Autowired
    private UserService userService;

    /* GROUP AMMOUNTS TESTS */

    @Test
    public void shouldFailWithWrongAccesGroupsAmmounts() throws Exception {
        final String mail = "shouldFailWithWrongAccesGroupsAmmounts@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.TEST_USER));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(GROUP_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().is4xxClientError());

    }

    @Test
    public void shouldOkGettingGroupsAmmounts() throws Exception {
        final String mail = "shouldOkGettingGroupsAmmounts@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.DEANERY));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(GROUP_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().isOk());

    }

    @Test
    public void shouldFailWithNoTokenGroupsAmmounts() throws Exception {
        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(GROUP_AMMOUNTS_ENDPOINT)).andExpect(status().is4xxClientError());

    }

    /* USERS ASSIGNED TESTS */

    @Test
    public void shouldFailWithWrongAccesRegisteredStudentsAmmount() throws Exception {
        final String mail = "shouldFailWithWrongAccesRegisteredStudentsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.TEST_USER));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_ASSIGNED_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().is4xxClientError());

    }

    @Test
    public void shouldOkGettingRegisteredStudentsAmmount() throws Exception {
        final String mail = "shouldOkGettingRegisteredStudentsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.DEANERY));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_ASSIGNED_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().isOk());

    }

    @Test
    public void shouldFailWithNoTokenRegisteredStudentsAmmount() throws Exception {
        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_ASSIGNED_AMMOUNTS_ENDPOINT)).andExpect(status().is4xxClientError());

    }

    /* USERS NO ASSIGNED TESTS */

    @Test
    public void shouldFailWithWrongAccesNoRegisteredStudentsAmmount() throws Exception {
        final String mail = "shouldFailWithWrongAccesNoRegisteredStudentsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.TEST_USER));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_NO_ASSIGNED_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().is4xxClientError());

    }

    @Test
    public void shouldOkGettingNoRegisteredStudentsAmmount() throws Exception {
        final String mail = "shouldOkGettingNoRegisteredStudentsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.DEANERY));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_NO_ASSIGNED_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().isOk());

    }

    @Test
    public void shouldFailWithNoTokenNoRegisteredStudentsAmmount() throws Exception {
        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_NO_ASSIGNED_AMMOUNTS_ENDPOINT)).andExpect(status().is4xxClientError());

    }

    /* USERS FULL ACCPTED TESTS */

    @Test
    public void shouldFailWithWrongAccessAcceptedStudentsAmmount() throws Exception {
        final String mail = "shouldFailWithWrongAccessAcceptedStudentsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.TEST_USER));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_ACCEPTED_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().is4xxClientError());

    }

    @Test
    public void shouldOkGettingAcceptedStudentsAmmount() throws Exception {
        final String mail = "shouldOkGettingAcceptedStudentsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.DEANERY));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_ACCEPTED_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().isOk());

    }

    @Test
    public void shouldFailWithNoTokenAcceptedStudentsAmmount() throws Exception {
        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_ACCEPTED_AMMOUNTS_ENDPOINT)).andExpect(status().is4xxClientError());

    }

    /* USERS PARTLY ACCPTED TESTS */

    @Test
    public void shouldFailWithWrongAccessPartlyAcceptedStudentsAmmount() throws Exception {
        final String mail = "shouldFailWithWrongAccessPartlyAcceptedStudentsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.TEST_USER));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_PARTLY_ACCEPTED_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().is4xxClientError());

    }

    @Test
    public void shouldOkGettingPartlyAcceptedStudentsAmmount() throws Exception {
        final String mail = "shouldOkGettingPartlyAcceptedStudentsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.DEANERY));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_PARTLY_ACCEPTED_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().isOk());

    }

    @Test
    public void shouldFailWithNoTokenPartlyAcceptedStudentsAmmount() throws Exception {
        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(USER_PARTLY_ACCEPTED_AMMOUNTS_ENDPOINT)).andExpect(status().is4xxClientError());

    }

    /* USERS FULL TAKEN GROUPS TESTS */

    @Test
    public void shouldFailWithWrongAccessFullGroupsAmmount() throws Exception {
        final String mail = "shouldFailWithWrongAccessFullGroupsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.TEST_USER));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(GROUP_FULL_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().is4xxClientError());

    }

    @Test
    public void shouldOkGettingFullGroupsAmmount() throws Exception {
        final String mail = "shouldOkGettingFullGroupsAmmount@StatisticsController.test";
        final User usr = this.userService.save(new User(null, null, mail, UserRoles.DEANERY));

        final String token = this.userService.login(usr).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(GROUP_FULL_AMMOUNTS_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().isOk());

    }

    @Test
    public void shouldFailWithNoTokenFullGroupsAmmount() throws Exception {
        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(GROUP_FULL_AMMOUNTS_ENDPOINT)).andExpect(status().is4xxClientError());

    }
}
